package ZipTie::Adapters::F5::BigIP::Disconnect;

use strict;
use warnings;
use 5.010_000;

use ZipTie::CLIProtocol;
use ZipTie::Response;
use ZipTie::Logger;

use Exporter qw(import);
our @EXPORT_OK = qw(disconnect);

my $LOGGER = ZipTie::Logger::get_logger();

# Adapter could be in 2 different places; tmsh, or Advanced Shell
#
# Here is the current logic for trying to back out of the session:
# 1. Initially send "exit", if the prompt contains "(tmos)#" continue
# 2. Send "quit" to exit tmsh shell, if the prompt does not contain "(tmos)#" continue
# 3. Send "exit" to exit the Advanced shell
# Session should now be terminated.
# If at any step above the prompt was empty after issuing a command it is concluded that
# the session is over.

sub disconnect {
	my $cli_protocol = shift;

	_send_exit($cli_protocol);

	$cli_protocol->disconnect();
}

sub _send_exit {
	my $cli_protocol = shift;

	my @responses = (
		ZipTie::Response->new('\(tmos\)#\s*$', \&_send_quit),
		ZipTie::Response->new('.*')
	);

	# Send the "exit" command
	$cli_protocol->send('exit');
	my $response = $cli_protocol->wait_for_responses(\@responses);

	# Based on the response of the device, determine the next interaction that should be executed.
	my $next_interaction;
	if( $response ) {
		$next_interaction = $response->get_next_interaction();
	} else {
		$LOGGER->fatal('Invalid response from device encountered!');
	}

	if( $next_interaction ) {
		return &$next_interaction( $cli_protocol );
	}

	return;
}

sub _send_quit {
	my $cli_protocol = shift;

	my @responses = (
		ZipTie::Response->new('(?<!\(tmos\))#\s*$', \&_send_exit),
		ZipTie::Response->new('\(tmos\)#\s*$', \&_send_quit),
		ZipTie::Response->new('.*')
	);

	# Send the "quit" command
	$cli_protocol->send('quit');
	my $response = $cli_protocol->wait_for_responses(\@responses);

	# Based on the response of the device, determine the next interaction that should be executed.
	my $next_interaction;
	if( $response ) {
		$next_interaction = $response->get_next_interaction();
	} else {
		$LOGGER->fatal('Invalid response from device encountered!');
	}

	if( $next_interaction ) {
		return &$next_interaction( $cli_protocol );
	}
	return;
}

1;
