#!/usr/bin/perl

use strict;
use warnings;
use 5.010_000;

use ZipTie::Adapters::PaloAlto::PANOS::AutoLogin qw(get_connection get_response_objects);
use ZipTie::Adapters::PaloAlto::PANOS::Utils qw(get_value_from_xml);
use ZipTie::Adapters::Utils qw(get_crep);
use ZipTie::Client;
use ZipTie::Discovery qw(get_connection_path get_devices add_or_update_devices);
use ZipTie::Logger;

my $ADAPTER = 'ZipTie::Adapters::PaloAlto::PANOS';

my $LOGGER = ZipTie::Logger::get_logger();

execute_discovery( $ARGV[0] );

sub execute_discovery {
	my ($management_server_ip) = @_;

	# Enable logging to file
	if ( $ENV{ADAPTER_LOG_TO_FILE} ) {
		$LOGGER->enable_logging_to_file( 'PANOS', 'PanoramaDiscovery', $management_server_ip );
	}

        # Check parameter is a valid ip
        my $ip_regex = get_crep('cipm');
        if($management_server_ip !~ /^$ip_regex$/){
                _fatal("Invalid IP address: $management_server_ip");
        }

	# Obtain a connection to the Ziptie API
	my $ziptie_client = ZipTie::Client->new();

	# Create a connection path object to use when connecting to the device
	my $connection_path = get_connection_path( $ziptie_client, $management_server_ip );

	my $connection = get_connection($connection_path);

	my %commands = (
		required => [
			[ show_devices_connected => 'type=op&cmd=<show><devices><connected></connected></devices></show>' ]
		],
		optional => []
	);

	my $responses = get_response_objects($connection, \%commands);
	my $show_devices_connected = $responses->{show_devices_connected};

	# Parse the IP addresses of the managed devices from the output of the show devices connected response
	my @device_ips;
	my @devices = $show_devices_connected->findnodes('/response/result/devices/entry');
        for my $device ( @devices ) {
		push @device_ips, get_value_from_xml( 'ip-address', $device );
	}

	my $existing_devices = get_devices( $ziptie_client, $management_server_ip );

	add_or_update_devices( $ziptie_client, $ADAPTER, $management_server_ip, $existing_devices, \@device_ips );

	$ziptie_client->logout();

	return;
}
