package ZipTie::Adapters::PaloAlto::PANOS::Applications;

use strict;
use warnings;
use 5.010_000;

use ZipTie::Adapters::PaloAlto::PANOS::AutoLogin qw(get_connection get_response_objects);
use ZipTie::Adapters::PaloAlto::PANOS::Parsers qw(parse_applications);
use ZipTie::Adapters::Utils qw(get_model_filehandle close_model_filehandle);

use ZipTie::ApplicationsClient;
use ZipTie::ApplicationsClient::Application;
use ZipTie::ApplicationsClient::ApplicationClassification;
use ZipTie::ApplicationsClient::ApplicationService;

use ZipTie::ConnectionPath;
use ZipTie::Logger;
use ZipTie::Model::XmlPrint;
use ZipTie::Typer;

my $LOGGER = ZipTie::Logger::get_logger();

sub invoke {
	my $connection_path_xml = shift;

	$LOGGER->debug('Starting GetApplications operation.');

	my ($connection_path) = ZipTie::ConnectionPath::from_xml($connection_path_xml);

	my $connection = get_connection($connection_path);

	$LOGGER->debug('Connection established');

	my %commands = (
		required => [
			[ get_predefined_applications => 'type=config&action=get&xpath=/config/predefined/application' ]
		],
		options => []
	);

	my $responses = get_response_objects($connection, \%commands);

	my $full_config = $responses->{get_predefined_applications};

	my $applications_client = ZipTie::ApplicationsClient->new();

	my $counter = 0;
	my @entries = $full_config->findnodes('/response/result');
	for my $entry ( @entries ) {
		my $applications = parse_applications($entry);
		next unless $applications;

		for my $parsed_application ( @{ $applications } ) {
			++$counter;

			#pull out the basic information
			my $name        = $parsed_application->{'ztApp:appName'};
			my $description = $parsed_application->{'ztApp:appDescription'};
			my $risk        = $parsed_application->{'ztApp:appRiskSeverity'};

			# Create the application object
			my $application = ZipTie::ApplicationsClient::Application->new(
				{
					adapterId              => 'ZipTie::Adapters::PaloAlto::PANOS',
					applicationName        => $name,
					applicationDescription => $description,
					riskSeverity           => $risk
				}
			);

			# Create application services
			for my $port ( @{ $parsed_application->{'ztApp:appNetworkPort'} } ) {

				# Schema currently only supports TCP and UDP
				my $protocol = $port->{'ztApp:protocol'};

				my ($port_low, $port_high);
				if( defined $port->{'ztApp:port'}->{'ztApp:portValue'} ) {
					$port_low = $port->{'ztApp:port'}->{'ztApp:portValue'};
				} else {
					$port_low  = $port->{'ztApp:port'}->{'ztApp:portRange'}->{'ztApp:portLow'};
					$port_high = $port->{'ztApp:port'}->{'ztApp:portRange'}->{'ztApp:portHigh'};
				}

				my $application_service = ZipTie::ApplicationsClient::ApplicationService->new( { protocol => $protocol, portLow => $port_low, portHigh => $port_high } );

				$application->add_application_service($application_service);
			}

			# Create application categorys
			for my $category ( @{ $parsed_application->{'ztApp:appClassification'} } ) {
				my $class_name  = $category->{'ztApp:classifierName'};
				my $class_value = $category->{'ztApp:classifierValue'};

				my $application_category = ZipTie::ApplicationsClient::ApplicationClassification->new( { classifierName => $class_name, classifierValue => $class_value } );

				$application->add_application_classification($application_category);
			}

			$applications_client->add_application($application);
		}
	}

	$LOGGER->debug("Commiting $counter applications to ZipTie.");
	$applications_client->commit();
	$LOGGER->debug('Get Applications Complete');

	return;
}

1;
